import { refNormalizeOptions } from '@calendair/core';
import { computed } from 'vue';
import { startOfDay } from 'date-fns/esm';

function HighlightPlugin(options) {
    const opts = computed(() => {
        const normalized = refNormalizeOptions(options, {
            style: {
                zIndex: '1',
                borderColor: '#9ae6b4',
                backgroundColor: '#f0fff4',
            }
        });
        return {
            ...normalized.value,
            highlights: normalized.value.highlights && typeof normalized.value.highlights !== 'function'
                ? normalized.value.highlights.map(startOfDay)
                : normalized.value.highlights
        };
    });
    const plugin = ctx => {
        patchTheme(ctx, opts);
    };
    return plugin;
}
function patchTheme(ctx, options) {
    ctx.theme.cellStyleInterceptors.value.push((day, state, prev) => {
        if (state.hovered ||
            state.inSelection ||
            state.inShadowSelection ||
            state.selected ||
            !state.currentMonth) {
            return prev;
        }
        if (!isHighlighted(day.date, options.value)) {
            return prev;
        }
        return {
            ...prev,
            ...options.value.style,
        };
    });
}
function isHighlighted(date, options) {
    if (!options.highlights) {
        return false;
    }
    if (typeof options.highlights === 'function') {
        return options.highlights(date);
    }
    return !!options.highlights.find(h => h.getTime() === date.getTime());
}

export default HighlightPlugin;
